//**********************************************************************************************
// Copyright (c) 2005, 2006 Kirk Haselden.
//
// File:                      CryptoTask.cs
// Purpose:               Encrypt and Decrypt text files
// DATE:                  1/18/2006 
// PROJECT:		       SSIS2K5 Samples
// PROGRAMMER:   Kirk Haselden
// COMMENTS:        
// TABS:		             3/3 retain tabs
//
// Disclaimer : This code is presented as sample code with no warranties.
//                   This code has not been thoroughly tested and should not be used in
//                   production systems. 
//
//**********************************************************************************************

#region using directives
using System;
using System.Text;
using System.Xml;
using System.Windows.Forms;
using Microsoft.SqlServer.Dts.Runtime;
using System.IO;
using System.Security.Cryptography;

#endregion

namespace SSIS2K5.Samples.Tasks.CryptoTask
{
   [
   DtsTask(
		TaskType = "DTS90",
      DisplayName="CryptoTask",
      IconResource="SSIS2K5.Samples.Tasks.CryptoTask.TaskIcon.ico",
      Description="A task for encrypting text files",
      UITypeName = "SSIS2K5.Samples.Tasks.CryptoTask.CryptoTaskUI, SSIS2K5.Samples.CryptoTaskUI, Version=1.0.0.1, Culture=Neutral, PublicKeyToken=844aba2619516093",
		TaskContact = "CryptoTask; Microsoft SQL Server Integration Services 2005, By Kirk Haselden; For Version 9.0 of Integration Services; (c) 2006 Kirk Haselden;http://www.CustomTask.com/CryptoTask")
   ]
   public sealed class CryptoTask : Task, IDTSComponentPersist
   {
      #region Task Specifics
         public enum EncryptOp { Encrypt, Decrypt }      
      #region Private Members
         private string m_password = "";
         private string m_source = "";
         private string m_dest = "";
         private EncryptOp m_operation;
         private bool m_overwriteDest = false;
         private Connections m_connections = null;
         private string CryptoTaskLogEntryDescription = "Logs the file and encryption operation performed";
      #endregion

      #region Public Properties

      // The symmetric key (password)
      public string Password
      {
         set
         {
            m_password = value;
         }
      }

      // SourceConnection manager
      public string SourceFileConnectionManager
      {
         get
         {
            return GetConnectionName(m_connections, m_source);
         }
         set
         {
            m_source = GetConnectionID(m_connections, value);
         }
      }

      // DestinationConnection manager
      public string DestinationFileConnectionManager
      {
         get
         {
            return GetConnectionName(m_connections, m_dest);
         }
         set
         {
            m_dest = GetConnectionID(m_connections, value);
         }
      }


      public EncryptOp EncryptionOperation
      {
         get
         {
            return m_operation;
         }
         set
         {
            m_operation = value;
         }
      }

      public bool OverwriteDestination
      {
         get
         {
            return m_overwriteDest;
         }
         set
         {
            m_overwriteDest = value;
         }
      }

      #endregion

      #endregion

      #region Task Section
      #region InitializeTask
      // 
      // Method called when the package with the task is loaded.
      public override void InitializeTask(Connections connections, VariableDispenser variableDispenser, IDTSInfoEvents events, IDTSLogging log, EventInfos eventInfos, LogEntryInfos logEntryInfos, ObjectReferenceTracker refTracker)
		{
         m_connections = connections;
         if (null != logEntryInfos)
			{				
            // Add a log entry info so users can filter the log entries of this type in the designer.
            // This creates an entry in the logging dialog box under details where you can enable the log entry.
				logEntryInfos.Add(LOGEVENTNAMES[(int)ENUMLOG.CRYPTOFILELOGENTRY], CryptoTaskLogEntryDescription, DTSLogEntryFrequency.Occasional);

			}

         // A sample event handler that does nothing.
         string[] parameterNames = new string[2] { "Parameter1", "Parameter2" };
         TypeCode[] parameterTypes = new TypeCode[2] { TypeCode.String, TypeCode.Int32 };
         string[] parameterDescriptions = new string[2] { "Parameter desc1.", "Parameter desc2" };
         Boolean allowEventHandlers = true;

         eventInfos.Add(
            "CustomEventName",
            "A custom event to show how they are created",
            allowEventHandlers,
            parameterNames,
            parameterTypes,
            parameterDescriptions
         );

		}
		#endregion

      // Task Validate method for checking if needed resources are available.
      // Ideally, if the task passes validation, it will always execute successfully, 
      // or at least with succeed with regard to finding needed resources.
		#region Validate
		public override DTSExecResult Validate(Connections connections, VariableDispenser variableDispenser, IDTSComponentEvents events, IDTSLogging log)
		{
         DTSExecResult result = DTSExecResult.Success;

         m_connections = connections;
         try
         {
            // Validate that the password exists, or else the encryption will encrypt with an empty string
            if (m_password.Length == 0)
            {
               events.FireError(15, "", "There is no password specified. Check the Package Protection Level to ensure that DontSaveSensitive is not selected.", "", 0);
            }

            if (connections != null && events != null)
            {
               // Check if connections are available and the right type
               ValidateConnectionManager(connections, events, GetConnectionName(m_connections, m_source), false, m_overwriteDest);
               ValidateConnectionManager(connections, events, GetConnectionName(m_connections, m_dest), true, m_overwriteDest);
            }
         }
         catch (Exception e)
         {
            if (null != events)
            {
               events.FireError(0, "", e.Message, "", 0);
            }
            result = DTSExecResult.Failure;
         }	

         return result;
		}

      // Helper function to validate Connection Managers
      private void ValidateConnectionManager(Connections connections, IDTSComponentEvents events, string sCMName, Boolean bDest, Boolean bOverwriteDest)
      {
                  if (!connections.Contains(sCMName))
                  {
                     StringBuilder sb = new StringBuilder("The specified source connection manager : " + sCMName + " does not exist.");
                     events.FireError(1, "", sb.ToString(), "", 0);
                  }
                  else // Connection Manager exists. Let's validate it.
                  {
                     // Is the Connection Manager a file or flatfile Connection Manager?
                     if (connections[sCMName].CreationName != "FILE" && connections[sCMName].CreationName != "FLATFILE")
                     {
                        // No, alert user with an error.
                        StringBuilder sb = new StringBuilder("The specified source connection manager : " + sCMName + " is not a file or flatfile connection manager.");
                        events.FireError(2, "", sb.ToString(), "", 0);
                     }
                     else // We have the correct type of Connection Manager. Now check the file to see if it is there.
                     {
                        // Get the file name
                        string sFile = (string)connections[sCMName].AcquireConnection(null);

                        // Check the file information
                        FileInfo fiSource = new FileInfo(sFile);

                        // Does the file even exist?
                        // If it doesn't exist and it's not the destination Connection Manager, let the user know.
                        if (!fiSource.Exists)
                        {
                           if( !bDest)
                           {                     
                              // Only fire a warning here because the file may become available later in the package.
                              StringBuilder sb = new StringBuilder("The specified connection manager : " + sCMName + " points to the following file which does not exist : " + sFile);
                              events.FireWarning(3, "", sb.ToString(), "", 0);
                           }                     
                        }
                        else if( bDest) // File Exists. If it is the destination Connection Manager, and the file does exist, we need to do some extra checking.
                        {
                           if( !bOverwriteDest ) // We're not supposed to overwrite the file
                           {
                              StringBuilder sb = new StringBuilder("The specified connection manager : " + sCMName + " points to an existing file  : " + sFile + " and Overwrite is set to False.");
                              events.FireError(4, "", sb.ToString(), "", 0);
                           }
                           
                           if( (fiSource.Attributes & FileAttributes.ReadOnly) != 0 ) // Just check to see that the file is not read only.
                           {
                              StringBuilder sb = new StringBuilder("The specified connection manager : " + sCMName + " points to an existing file  : " + sFile + " that is read only.");
                              events.FireError(5, "", sb.ToString(), "", 0);
                           }                     
                        }
                     }
                  }

         return;
      }


		#endregion

		#region Execute
		public override DTSExecResult Execute(Connections connections, VariableDispenser variableDispenser, IDTSComponentEvents events, IDTSLogging log, object transaction)
		{
         // Set up for success.
         DTSExecResult result = DTSExecResult.Success;

         // Files to use
         string sourceFileName = "";
         string destFileName = "";

         // Check if the client wants to continue receiving events.
         Boolean fireAgain = true;

         // Fire the custom event
         object[] objArgs = new object[2] {"Parameter1 Value", 42};
         events.FireCustomEvent(
            "CustomEventName",
            "Custom event fired",
				ref objArgs, "", ref fireAgain
          );

         m_connections = connections;
         try
         {
            // Fire starting progress event.
            events.FireProgress("Starting to execute the CryptoTask", 0, 0, 0, "", ref fireAgain);

            // Get the source file name string
            ConnectionManager cm = connections[m_source];
            if (cm != null)
            {
               // Get the file name
               sourceFileName = (string)cm.AcquireConnection(null);
            }

            // Get the destination file name string
            cm = connections[m_dest];
            if (cm != null)
            {
               destFileName = (string)cm.AcquireConnection(null);
            }

            // Write some information about the execution.
            StringBuilder sb = new StringBuilder();
            sb.Append(m_operation == EncryptOp.Decrypt ? "Decrypting : " : "Encrypting : ");
            sb.Append(sourceFileName + " to the file : " + destFileName);

            WriteLog(log, LOGEVENTNAMES[(int)ENUMLOG.CRYPTOFILELOGENTRY], sb.ToString());


            //
            // Open the source file, encrypt/decrypt it and write it out to the destination file
            //

            // Read the file into a string
            TextReader trSource = System.IO.File.OpenText(sourceFileName);
            string sSource = trSource.ReadToEnd();

            // Release the file
            trSource.Close();

            // Encrypt or decrypt the text
            string sDest = "";                 // encrypted/decrypted text
            string initVector = "*fsy&$lKmz@-+x5/"; // must be 16 bytes

            // Append plain text to a random salt value
            RijndaelEnhanced rijndaelKey =
                new RijndaelEnhanced(this.m_password, initVector);

            // Encrypt/Decrypt based on property setting
            if (m_operation == EncryptOp.Encrypt)
            {
               sDest = rijndaelKey.Encrypt(sSource);
            }
            else
            {
               sDest = rijndaelKey.Decrypt(sSource);
            }

            // Open the file
            FileInfo fiDest = new FileInfo(destFileName);
            StreamWriter swDest = fiDest.CreateText();

            // Write to the file
            swDest.Write(sDest);

            // Flush the contents
            swDest.Flush();

            // Release the file
            swDest.Close();

            // Fire completion progress event.
            // This is the minimum progress event logging that should always be done.            
            events.FireProgress("Completed execution of the CryptoTask", 100, 0, 0, "", ref fireAgain);
         }
         catch (Exception e)
         {
            if (null != events)
            {
               events.FireError(100, "", "There was an error with the encryption. Check the password, it must be the same as the one used to encrypt the file. More detailed information is available in the following error." , "", 0);
               events.FireError(0, "", e.Message, "", 0);
            }
            result = DTSExecResult.Failure;
         }
			return result;
		}

      private void MessageBox(string value)
      {
         System.Windows.Forms.MessageBox.Show(value);
      }
		#endregion

		#region Execution Value
		public override object ExecutionValue
		{
			get
         { 
               return (object)0; 
         }
		}
		#endregion
		#endregion
		
      #region IDTSComponentPersist

		#region String Constants for node names
		private const string PERSIST_CONST_ELEMENT	               = "CryptoTaskData";
		private const string PERSIST_CONST_PASSWORD           = "Password";
      private const string PERSIST_CONST_SENSITIVE              = "Sensitive";
      private const string PERSIST_CONST_SOURCE                 = "Source";
      private const string PERSIST_CONST_DEST                      = "Dest";
      private const string PERSIST_CONST_ENCRYPTOP           = "EncryptOp";
      private const string PERSIST_CONST_OVERWRITE           = "OverwriteDest";
      public const string PERSIST_CONST_ENCRYPT                = "Encrypt";
      public const string PERSIST_CONST_DECRYPT                = "Decrypt";

      private string[] ENCRYPTOPNAMES = { PERSIST_CONST_ENCRYPT, PERSIST_CONST_DECRYPT };
		#endregion

		#region Save To XML
		public void SaveToXML(XmlDocument doc, IDTSInfoEvents events)
		{
         try
         {
            // Create an root node for the data
            XmlElement rootElement = doc.CreateElement("", PERSIST_CONST_ELEMENT, "");
            doc.AppendChild(rootElement);

            // Save the source file connection manager name
            XmlAttribute sourceAttr = doc.CreateAttribute(PERSIST_CONST_SOURCE);
            sourceAttr.Value = m_source;
            rootElement.Attributes.Append(sourceAttr);

            // Save the destination file connection manager name
            XmlAttribute destAttr = doc.CreateAttribute(PERSIST_CONST_DEST);
            destAttr.Value = m_dest;
            rootElement.Attributes.Append(destAttr);

            // Save the encryption option value
            XmlAttribute encryptAttr = doc.CreateAttribute(PERSIST_CONST_ENCRYPTOP);
            encryptAttr.Value = ENCRYPTOPNAMES[(int)m_operation];
            rootElement.Attributes.Append(encryptAttr);


            // Save the encryption option value
            XmlAttribute overwriteAttr = doc.CreateAttribute(PERSIST_CONST_OVERWRITE);
            overwriteAttr.Value = m_overwriteDest.ToString();
            rootElement.Attributes.Append(overwriteAttr);

            // Save the password 
            XmlElement pswdElement = doc.CreateElement("", PERSIST_CONST_PASSWORD, "");
            rootElement.AppendChild(pswdElement);

            // Set the password element as sensitive so that the runtime will encrypt it.
            pswdElement.InnerText = m_password;
            XmlAttribute attr = doc.CreateAttribute(PERSIST_CONST_SENSITIVE);
            attr.Value = "1";
            pswdElement.Attributes.Append(attr);
         }
         catch (Exception e)
         {
            if (null != events)
            {
               events.FireError(0, "", e.Message, "", 0);
            }
         }

		}
		#endregion         

      #region Load From XML
      public void LoadFromXML(XmlElement rootNode, IDTSInfoEvents events)
		{
         try
         {
			   //Set properties to their default values;
			   m_password = "";
			   
            // Load the root node for the data
            // This should be the root data node created above.
            // If not, throw and show. 
            if (rootNode.Name != PERSIST_CONST_ELEMENT)
				   throw new Exception("Invalid Start Task data node");

            // Load the Connection and other properties
            m_source = rootNode.GetAttribute(PERSIST_CONST_SOURCE);
            m_dest = rootNode.GetAttribute(PERSIST_CONST_DEST);
            m_operation = (rootNode.GetAttribute(PERSIST_CONST_ENCRYPTOP) == ENCRYPTOPNAMES[(int)EncryptOp.Encrypt] ? 
               EncryptOp.Encrypt : EncryptOp.Decrypt);            
            
            // Default to _don't overwrite_ if value is anything other than true.
            string overwrite = rootNode.GetAttribute(PERSIST_CONST_OVERWRITE);
            m_overwriteDest = (overwrite != "True" ? false : true);
            
			   // Load the password
            // Find the password childnode
            foreach(XmlNode childNode in rootNode.ChildNodes)
            {
               if(childNode.Name == PERSIST_CONST_PASSWORD)
               {
                  m_password = childNode.InnerText;
               }
            }
         }
         catch( Exception e)
         {
            if (null != events)
            {
               // Shows the exception as an error with the message.
               events.FireError(0, "", e.Message, "", 0);
            }          
         }
		}
		#endregion
      
      #endregion

      #region Logging
      // List of the log events that could potentially be logged from this task
      // Used to add log event entries into the LogEventInfos collection in InitializeTask
		private static string[] LOGEVENTNAMES = 
		{ 
			"CryptoFileLogEntry"			
		};

		// Enumerate values to index into the LOGEVENTNAMES above
		private enum ENUMLOG
		{
			CRYPTOFILELOGENTRY
		}
      // A simplified method for logging.
      // Does not require the additional overwritten parameters.
      private void WriteLog(IDTSLogging log, string logEventName, string message)
      {
         if (log != null && log.Enabled)
         {
            byte[] nothing = null;
            log.Log(logEventName, null, null, null, null, null, message, DateTime.Now, DateTime.Now, 0, ref nothing);
         }
      }
		#endregion
   }
}
