using System;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.Globalization;


// A class for retrieving the properties of an image file.
namespace SSIS2K5.Samples.Transforms.ImageFileSrc
{
   public class ExifReader
   {

      #region Private Constants
      // Property IDs for the EXIF properties
      const int MAKE = 271;
      const int MODEL = 272;
      const int ORIENTATION = 274;
      const int DATETIME = 306;
      const int ARTIST = 315;
      const int SHUTTERSPEED = 33434;
      const int FSTOP = 33437;
      const int EXPOSUREPROGRAM = 34850;
      const int ISOID = 34855;
      const int SUBJECTDISTANCE = 37382;
      const int METERINGMODE = 37383;
      const int FLASH = 37385;
      const int FOCALLENGTH = 37386;
      const int EXPOSUREMODE = 41986;
      const int WHITEBALANCE = 41987;
      const int SHARPNESS = 41994;
      const int IMAGEWIDTH = 256;
      const int IMAGELENGTH = 257;
      const int BITSPERSAMPLE = 258;
      const int COMPRESSION = 259;
      #endregion

      #region Privates
      DateTimeFormatInfo m_dateTimeFormat = new DateTimeFormatInfo();
      private ASCIIEncoding m_asciiEnc;      
      private Image m_image;
      private int[] m_PropertyIdList;
      private string m_fileName = "";
      #endregion

      #region Exif Properties
      // Constructor
      public ExifReader(string fileName)
      {
         m_fileName = fileName;
         m_image = Image.FromFile(fileName);
         
         try
         {
            m_PropertyIdList = m_image.PropertyIdList;
            m_asciiEnc = new ASCIIEncoding();
         }
         catch (Exception ex)
         {
            // Failed to open the file.
            throw (ex);
         }

      }

      public string GetProperty(string propName)
      {
         switch (propName)
         {
            case "FILENAME": return FileName;
            case  "ARTIST": return Artist;
            case "MAKE": return Make;
            case "MODEL": return Model;
            case "SHUTTERSPEED": return ShutterSpeed;
            case "DATETIME": return DateTime;
            case "FSTOP": return FStop;
            case "EXPOSUREPROGRAM": return ExposureMode;
            case "ISO": return ISO;
            case "IMAGEWIDTH": return ImageWidth;
            case "IMAGELENGTH": return ImageLength;
            case "BITSPERSAMPLE": return BitsPerSample;
            case "COMPRESSION": return Compression;
         }
         return "NA";
      }

      public string FileName
      {
         get
         {
            return m_fileName;
         }
      }

      public string ImageWidth
      {         
         get
         {
            try
            {
               return m_image.Width.ToString();
            }
            catch (Exception) { }
            return "";
         }
      }
      public string ImageLength
      {
         get
         {
            try
            {
               return m_image.Height.ToString();
            }
            catch (Exception){}
            return "";
         }
      }
      public string BitsPerSample
      {
         get
         {
            try
            {
               return m_image.PixelFormat.ToString();
            }
            catch (Exception) { }
            return "";
         }
      }
      public string Compression
      {
         get
         {
            try
            {
               PropertyItem propItem = m_image.GetPropertyItem(COMPRESSION);
               string strPropVal = BitConverter.ToInt16(propItem.Value, 0).ToString();
               return strPropVal;
            }
            catch (Exception) { }
            return "";
         }
      }
      public string Make
      {
         get
         {
            try
            {
               PropertyItem propItem = m_image.GetPropertyItem(MAKE);
               string decodedstring = m_asciiEnc.GetString(propItem.Value);
               return decodedstring;
            }
            catch (Exception) { }
            return "";
         }
      }
      public string Model
      {
         get
         {
            try
            {
               PropertyItem propItem = m_image.GetPropertyItem(MODEL);
               string decodedstring = m_asciiEnc.GetString(propItem.Value);
               return decodedstring;
            }
            catch (Exception) { }
            return "";
         }
      }
      public string Orientation
      {
         get
         {
            try
            {
               PropertyItem propItem = m_image.GetPropertyItem(ORIENTATION);
               string strPropVal = BitConverter.ToInt16(propItem.Value, 0).ToString();
               if (strPropVal == "1") return "Correct";
               if (strPropVal == "6") return "Must rotate Right";
               if (strPropVal == "8") return "Must rotate Left";
               return strPropVal;
            }
            catch (Exception) { }
            return "";
         }
      }
      public string DateTime
      {
         get
         {
            try
            {
               PropertyItem propItem = m_image.GetPropertyItem(DATETIME);
               string decodedstring = m_asciiEnc.GetString(propItem.Value);
               return decodedstring;
            }
            catch (Exception) { }
            return "";
         }
      }
      public string Artist
      {
         get
         {
            try
            {
               PropertyItem propItem = m_image.GetPropertyItem(ARTIST);
               string decodedstring = m_asciiEnc.GetString(propItem.Value);
               return decodedstring;
            }
            catch (Exception) { }
            return "";
         }
      }
      public string ShutterSpeed
      {
         get
         {
            try
            {               
               PropertyItem propItem = m_image.GetPropertyItem(SHUTTERSPEED);
               string strPropVal = BitConverter.ToInt16(propItem.Value, 0).ToString();
               string propVal = BitConverter.ToInt16(propItem.Value, 4).ToString();
               // Funky format, turn 10/2500 into 1/250
               if (strPropVal.EndsWith("0") && propVal.EndsWith("0"))
               {
                  strPropVal = strPropVal.Substring(0, strPropVal.Length - 1);
                  propVal = propVal.Substring(0, propVal.Length - 1);
               }
               //Turn 8/1 into 8
               if (propVal == "1") return strPropVal;
               return strPropVal + "/" + propVal;
            }
            catch (Exception) { }
            return "";
         }
      }
      public string FStop
      {
         get
         {
            try
            {
               PropertyItem propItem = m_image.GetPropertyItem(FSTOP);
               string strPropVal = BitConverter.ToInt16(propItem.Value, 0).ToString();
               if (strPropVal.Length == 3) strPropVal = strPropVal.Substring(0, 2);
               if (strPropVal != "22" && strPropVal != "32" && !strPropVal.StartsWith("1"))
                  strPropVal = strPropVal.Substring(0, strPropVal.Length - 1) + "." + strPropVal.Substring(strPropVal.Length - 1, 1);
               strPropVal = strPropVal.Replace(".0", "");
               return strPropVal;
            }
            catch (Exception) { }
            return "";
         }
      }
      public string ExposureProgram
      {
         get
         {
            try
            {
               PropertyItem propItem = m_image.GetPropertyItem(EXPOSUREPROGRAM);
               string strPropVal = BitConverter.ToInt16(propItem.Value, 0).ToString();
               if (strPropVal == "0") return "Not Defined";
               if (strPropVal == "1") return "Manual";
               if (strPropVal == "2") return "Program";
               if (strPropVal == "3") return "Aperture Priority";
               if (strPropVal == "4") return "Shutter Priority";
               if (strPropVal == "5") return "Creative Program";
               if (strPropVal == "6") return "Action Program";
               if (strPropVal == "7") return "Portrait Mode";
               if (strPropVal == "8") return "Landscape Mode";
               return strPropVal;
            }
            catch (Exception) { }
            return "";
         }
      }
      public string ISO
      {
         get
         {
            try
            {
               PropertyItem propItem = m_image.GetPropertyItem(ISOID);
               string strPropVal = BitConverter.ToInt16(propItem.Value, 0).ToString();
               return strPropVal;
            }
            catch (Exception) { }
            return "";
         }
      }
      public string SubjectDistance
      {
         get
         {
            try
            {
               PropertyItem propItem = m_image.GetPropertyItem(SUBJECTDISTANCE);
               string strPropVal = BitConverter.ToInt16(propItem.Value, 0).ToString();
               return strPropVal;
            }
            catch (Exception) { }
            return "";
         }
      }
      public string MeteringMode
      {
         get
         {
            try
            {
               PropertyItem propItem = m_image.GetPropertyItem(METERINGMODE);
               string strPropVal = BitConverter.ToInt16(propItem.Value, 0).ToString();
               if (strPropVal == "1") return "Average Metering";
               if (strPropVal == "2") return "Center Weighted Average Metering";
               if (strPropVal == "3") return "Spot Metering";
               if (strPropVal == "4") return "MultiSpot Metering";
               if (strPropVal == "5") return "Matrix Metering";
               if (strPropVal == "6") return "Partial Metering";

               return "Unknown";
            }
            catch (Exception) { }
            return "";
         }
      }
      public string Flash
      {
         get
         {
            try
            {
               PropertyItem propItem = m_image.GetPropertyItem(FLASH);
               string strPropVal = BitConverter.ToInt16(propItem.Value, 0).ToString();
               if (strPropVal == "0") return "No Flash";
               if (strPropVal == "1") return "Flash";
            }
            catch (Exception) { }
            return "";
         }
      }
      public string FocalLength
      {
         get
         {
            try
            {
               PropertyItem propItem = m_image.GetPropertyItem(FOCALLENGTH);
               string strPropVal = BitConverter.ToInt16(propItem.Value, 0).ToString();
               return strPropVal;
            }
            catch (Exception) { }
            return "";
         }
      }
      public string ExposureMode
      {
         get
         {

            try
            {
               PropertyItem propItem = m_image.GetPropertyItem(EXPOSUREMODE);
               string strPropVal = BitConverter.ToInt16(propItem.Value, 0).ToString();
               if (strPropVal == "0") return "Auto Exposure";
               if (strPropVal == "1") return "Manual Exposure";
               if (strPropVal == "2") return "Auto Bracket";
               if (strPropVal == "3") return "Reserved";
               return strPropVal;
            }
            catch (Exception) { }
            return "";
         }
      }
      public string WhiteBalance
      {
         get
         {
            try
            {
               PropertyItem propItem = m_image.GetPropertyItem(WHITEBALANCE);
               string strPropVal = BitConverter.ToInt16(propItem.Value, 0).ToString();
               if (strPropVal == "0") return "Auto";
               if (strPropVal == "1") return "Manual";
            }
            catch (Exception) { }
            return "";
         }
      }
      public string Sharpness
      {
         get
         {
            try
            {
               PropertyItem propItem = m_image.GetPropertyItem(SHARPNESS);
               string strPropVal = BitConverter.ToInt16(propItem.Value, 0).ToString();
               if (strPropVal == "0") return "Normal";
               if (strPropVal == "1") return "Soft";
               if (strPropVal == "2") return "Hard";
               return strPropVal;
            }
            catch (Exception) { }
            return "";
         }
      }
      #endregion
   
   } // End ExifReader
}