using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

// TODO : Support other configuration types

namespace ConfigurationsEditor
{
    public partial class ConfigEditor : Form
    {
        #region PrivateMembers
        private string currentFile;
        private Boolean amEditing = false;
        #endregion

        #region Defines
        static string DefaultTitleBar = "SQL Server Integration Services Configurations Editor";
        static string LoadedTitleBar = "ConfigEdit : ";
        #endregion

        #region Constructors ========================
        public ConfigEditor()
        {
            InitializeComponent();
            EnableActiveFileMenuItems(false);
        }
        #endregion

        #region Helpers ========================

        // Check with user to see if they really want to exit        
        private Boolean ShouldLeaveEdit(string appendString)
        {
            // Check to see if the dirty bit is set.

            if (amEditing && configEditControl.Dirty)
            {
                //Check to see if user wishes to save work.
                DialogResult dr = MessageBox.Show("You have made changes. " + appendString , "Package Configurations Editor", MessageBoxButtons.OKCancel);
                if (dr == DialogResult.Cancel)
                {
                    return false;
                }
            }
            return true;
        }

        // Show the OpenFile dialog to get the name of the configuration to open.
        private string GetOpenFileName()
        {
            System.Windows.Forms.OpenFileDialog fd = new System.Windows.Forms.OpenFileDialog();

            fd.CheckFileExists = true;
            fd.AddExtension = true;
            fd.CheckPathExists = true;
            fd.DefaultExt = ".dtsconfig";
            fd.Filter = "Configuration files (*.dtsconfig)|*.dtsconfig|XML files (*.xml)|*.xml";
            fd.RestoreDirectory = true;
            fd.ValidateNames = true;
            fd.Title = "Select XML Configuration File";
            fd.ShowHelp = false;
            fd.Multiselect = false;
            fd.ShowDialog(this);

            return fd.FileName;
        }

        // Show the SaveFile dialog to save the configuration.
        // Returns an empty string when cancelled.
        private string GetSaveFileName( string Title )
        {
            System.Windows.Forms.SaveFileDialog fd = new System.Windows.Forms.SaveFileDialog();

            fd.CheckFileExists = false;
            fd.AddExtension = true;
            fd.CheckPathExists = true;
            fd.DefaultExt = ".dtsconfig";
            fd.Filter = "Configuration files (*.dtsconfig)|*.dtsconfig|XML files (*.xml)|*.xml";
            fd.RestoreDirectory = true;
            fd.ValidateNames = true;
            fd.Title = Title;;
            fd.ShowHelp = false;
            DialogResult dr = fd.ShowDialog(this);
            if (dr == DialogResult.Cancel)
                return "";
            
            return fd.FileName;
        }

        private void EnableActiveFileMenuItems(Boolean enable)
        {
            SaveConfig.Enabled = enable;
            SaveConfigAs.Enabled = enable;
            CloseConfig.Enabled = enable;
        }

        // Load a configuration.
        private Boolean LoadConfigurationSetup()
        {
            currentFile = GetOpenFileName();
            if (currentFile.Length < 3)
                return false;
            this.tbConfigSource.Text = currentFile;
            amEditing = true;
            configEditControl.Visible = true;
            EnableActiveFileMenuItems(true);
            configEditControl.Dirty = false;

            // Show file loaded in titlebar
            Text = LoadedTitleBar + currentFile;
            return true;
        }

        private Boolean CreateConfiguration()
        {
            currentFile = GetSaveFileName("Create New XML Configuration");
            if (currentFile.Length < 3)
                return false;
            this.tbConfigSource.Text = currentFile;
            amEditing = true;
            configEditControl.Visible = true;
            EnableActiveFileMenuItems(true);
            configEditControl.Dirty = false;
            configEditControl.CreateNewXMLConfiguration();
            // Show file created in titlebar
            Text = LoadedTitleBar + currentFile;

            return true;
        }

        // Close the currently editing configuration
        private void CloseConfigurationSetup()
        {
            currentFile = "";
            this.tbConfigSource.Text = currentFile;
            configEditControl.Visible = false;
            EnableActiveFileMenuItems(false);
            
            // Clear out the configurations grid.
            configEditControl.Clear();
            amEditing = false;

            // Set default titlebar
            Text = DefaultTitleBar;
        }

        #endregion

        #region MenuItemEventHandlers ========================
        //
        // The Menu Item Event Handlers
        //

        // Load a configuration file.
        private void loadConfigurationMenuItem_Click(object sender, EventArgs e)
        {
            // Warn the user that loading a new configuration will close the existing dirty file.
            if (ShouldLeaveEdit("Are you sure you want to load a different configuration?"))
            {
                // Close the open file
                if (amEditing)
                {
                    CloseConfigurationSetup();
                }
            }

            // Load a configuration.
            if (LoadConfigurationSetup())
            {
                configEditControl.LoadXMLConfiguration(currentFile);
            }
        }

        // Save the configuration with a new name.
        private void saveConfigurationAsMenuItem_Click(object sender, EventArgs e)
        {
            if (!amEditing)
                return;
            string tempFileName = GetSaveFileName("Save XML Configuration File");
            if (tempFileName.Length > 3)
            {
                currentFile = tempFileName;
                this.tbConfigSource.Text = currentFile;
            }
            configEditControl.SaveXMLConfiguration(currentFile);
        }

        // Close the currently opened configuration.
        private void closeConfigurationMenuItem_Click(object sender, EventArgs e)
        {
            if (ShouldLeaveEdit("Are you sure you want to close the current file?") == true)
            {
                // Close the file
                CloseConfigurationSetup();
            }
        }

        private void SaveConfig_Click(object sender, EventArgs e)
        {
            if (!amEditing)
                return;

            configEditControl.SaveXMLConfiguration(currentFile);
        }

        private void NewConfigMenuItem_Click(object sender, EventArgs e)
        {
            // Warn the user that creating a new configuration will close the existing dirty file.
            if (ShouldLeaveEdit("Are you sure you want to create a new configuration?"))
            {
                // Close the open file
                if (amEditing)
                {
                    CloseConfigurationSetup();
                }
            }

            // Load a configuration.
            if (CreateConfiguration())
            {
                configEditControl.CreateNewXMLConfiguration();
            }
        }                

        // Exit the application
        private void exitMenuItem_Click(object sender, EventArgs e)
        {
            if(ShouldLeaveEdit("Are you sure you want to exit?") == true)
                this.Close();
        }

        //Show the About box
        private void aboutMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox ab = new AboutBox();
            ab.Show();
        }
        #endregion


    }
}